package kr.co.uracle.m_push_plugin.mpush;


import static kr.co.uracle.m_push_plugin.push.PushUiHandler.setCallback;

import android.app.Activity;
import android.content.Context;
import android.util.Log;
import androidx.annotation.NonNull;
import android.os.Handler;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import m.client.push.library.PushManager;
import m.client.push.library.common.PushConstants;
import m.client.push.library.utils.PushUtils;

/**
 * MPushPlugin
 */
public class MPushPlugin implements FlutterPlugin, ActivityAware, MethodCallHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    public static MethodChannel channel;
    private Context context;
    private Activity activity;

    public MPushPlugin() {
    }
    public void onAttachedToEngine(Context context1, BinaryMessenger messenger) {
        context = context1;
        channel = new MethodChannel(messenger, "m_push_plugin");
        channel.setMethodCallHandler(this);
    }
    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "m_push_plugin");
        channel.setMethodCallHandler(this);
        context = flutterPluginBinding.getApplicationContext();
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        Log.e("call_method ::", call.method);
        if (call.method.equals("getPlatformVersion")) {
            result.success("Android " + android.os.Build.VERSION.RELEASE);
        } else if (call.method.equals("registerServiceAndUser")) {
            final JSONObject params1 = new JSONObject();
            try {
                params1.put(PushConstants.KEY_CUID, call.argument("cuid"));
                params1.put(PushConstants.KEY_CNAME, call.argument("name"));
            } catch (JSONException e) {
                e.printStackTrace();
            }
            if (PushUtils.checkNetwork(context)) {
                setCallback(result);
                new Handler().post(() -> {
                    // TODO Auto-generated method stub
                    try {
                        PushManager.getInstance().registerServiceAndUser(context, params1);
                    } catch (Exception e) {
                        result.error("ERROR", "Fail", e);
                    }
                });
            } else {
                result.error("ERROR", "network error", "network is not connected.");
            }

        } else if(call.method.equals("registerUser")){
            JSONObject returnValue = new JSONObject();
            try {
                returnValue.put("status", "SUCCESS");
                returnValue.put("message", "registerServiceAndUser를 사용하시기 바랍니다.");
            } catch (JSONException e) {
                throw new RuntimeException(e);
            }
            result.success(toMap(returnValue));

        }else if(call.method.equals("registerService")){
            JSONObject returnValue = new JSONObject();
            try {
                returnValue.put("status", "SUCCESS");
                returnValue.put("message", "registerServiceAndUser를 사용하시기 바랍니다.");
            } catch (JSONException e) {
                throw new RuntimeException(e);
            }
            result.success(toMap(returnValue));

        }else if (call.method.equals("unregisterService")) {
            if (PushUtils.checkNetwork(context)) {
                setCallback(result);
                new Handler().post(() -> {
                    // TODO Auto-generated method stub
                    PushManager.getInstance().unregisterPushService(context.getApplicationContext());
                });
            } else {
                result.error("ERROR", "network error", "network is not connected.");
            }
        } else if (call.method.equals("unregisterUser")) {
            final JSONObject params1 = new JSONObject();
            try {
                params1.put(PushConstants.KEY_CUID, "GUEST");
                params1.put(PushConstants.KEY_CNAME, "GUEST");
            } catch (JSONException e) {
                e.printStackTrace();
            }
            if (PushUtils.checkNetwork(context)) {
                setCallback(result);
                new Handler().post(() -> {
                    // TODO Auto-generated method stub
                    PushManager.getInstance().unregisterPushUser(context.getApplicationContext(), params1);
                });
            } else {
                result.error("ERROR", "network error", "network is not connected.");
            }
        } else if (call.method.equals("sendNotification")) {
            Log.e("sendNotification", call.arguments.toString());
            String ext = "메세지 테스트";
            if (PushUtils.checkNetwork(context)) {
                final String finalMessage = call.argument("message");
                final String cuid = call.argument("cuid");
                final String serviceCode = call.argument("serviceCode");
                JSONObject object1 = new JSONObject();
                try {
                    object1.put("title", "일반 알림");
                    object1.put("body", finalMessage);
                } catch (JSONException e) {
                    throw new RuntimeException(e);
                }
                new Handler().post(() -> {
                    PushManager.getInstance().sendPushMessage(context.getApplicationContext(), object1.toString(), ext, cuid,serviceCode );
                });
            } else {
                result.error("ERROR", "network error", "network is not connected.");
            }

        } else if(call.method.equals("readNotification")){
            if (PushUtils.checkNetwork(activity)) {
                setCallback(result);
                new Handler().post(() -> {
                    PushManager.getInstance().pushMessageReadConfirm(context.getApplicationContext(), call.argument("notification"));
                });
            } else {
                result.error("ERROR", "network error", "network is not connected.");
            }
        } else if(call.method.equals("pushMessageReceiveConfirm")){
            if (PushUtils.checkNetwork(activity)) {
                setCallback(result);
                new Handler().post(() -> {
                    PushManager.getInstance().pushMessageReceiveConfirm(context.getApplicationContext(), call.argument("jsonString"));
                });
            } else {
                result.error("ERROR", "network error", "network is not connected.");
            }
        } else if(call.method.equals("registerGroup")){
            if (PushUtils.checkNetwork(activity)) {
                setCallback(result);
                PushManager.getInstance().registerUserGroup(context.getApplicationContext(), call.argument("groupNum"));
            } else {
                result.error("ERROR", "network error", "network is not connected.");
            }
        } else if(call.method.equals("unregisterGroup")){
            if (PushUtils.checkNetwork(activity)) {
                setCallback(result);
                new Handler().post(() -> {
                    PushManager.getInstance().unregisterUserGroup(context.getApplicationContext(), call.argument("groupNum"));
                });
            } else {
                result.error("ERROR", "network error", "network is not connected.");
            }
        } else if (call.method.equals("notificationCenterGetBadgeNumber")){
            JSONObject returnValue = new JSONObject();
            try {
                returnValue.put("status", "SUCCESS");
                returnValue.put("message", "iOS ONLY.");
            } catch (JSONException e) {
                throw new RuntimeException(e);
            }
            result.success((returnValue));
        } else if(call.method.equals("remoteSetBadgeNumber")){
            JSONObject returnValue = new JSONObject();
            try {
                returnValue.put("status", "SUCCESS");
                returnValue.put("message", "iOS ONLY.");
            } catch (JSONException e) {
                throw new RuntimeException(e);
            }
            result.success(toMap(returnValue));
        } else if(call.method.equals("setEnabled")){
            Log.d("setEnabled","iOS Only");
        } else if (call.method.equals("onNotification")){
            String pushJsonObject = activity.getIntent().getStringExtra("JSON");
            if (pushJsonObject == null){
                pushJsonObject = "";
            }
            JSONObject returnValue = new JSONObject();
            try {
                returnValue.put("status", "SUCCESS");
                returnValue.put("payload", pushJsonObject);
            } catch (JSONException e) {
                throw new RuntimeException(e);
            }
            result.success(toMap(returnValue));
        } else {
            result.notImplemented();
        }
    }
    public static Map<String, Object> toMap(JSONObject object) {
        Map<String, Object> map = new HashMap<>();
        Iterator<String> keysItr = object.keys();
        while(keysItr.hasNext()) {
            String key = keysItr.next();
            Object value;
            try {
                value = object.get(key);
            } catch (JSONException e) {
                throw new RuntimeException(e);
            }
            map.put(key, value);
        }
        return map;
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }


    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        activity = binding.getActivity();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {

    }
}
