import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:m_push_plugin/m_push_plugin.dart';

void main() => runApp(const MyApp());

class MyApp extends StatelessWidget {
  const MyApp({super.key});

  @override
  Widget build(BuildContext context) {
    return const MaterialApp(
      title: 'Cupertino Demo',
      home: CupertinoExample(),
    );
  }
}

class CupertinoExample extends StatefulWidget {
  const CupertinoExample({super.key});

  @override
  State<CupertinoExample> createState() => _CupertinoState();
}

class _CupertinoState extends State<CupertinoExample> {
  final mPush = MPushPlugin();
  var _result = '결과';
  var _noti;

  @override
  void initState() {
    // TODO: implement initState
    super.initState();

    //[Android]앱이 종료된 경우 noti 데이터 전달
    mPush.getOnNotification().then((value) {
      _noti = value["payload"];
      if (_noti != "") {
        setResult("ReceivedHandler", value.toString());
      }
    });
    // noti data 전달
    mPush.setDidReceiveNotification((notification) {
      _noti = notification["payload"];
      setResult("didReceiveNotification", notification.toString());
    });
  }

  void setResult(String title, String? value) {
    setState(() {
      _result = title;
      _result += " result \n\n";
      if (value != null) {
        _result += value!;
      } else {
        _result += "Null Value";
      }
    });
  }

  void _showAlertDialog(BuildContext context, String body) {
    showCupertinoModalPopup<void>(
      context: context,
      builder: (BuildContext context) => CupertinoAlertDialog(
        title: const Text('결과'),
        content: Text(body),
        actions: <CupertinoDialogAction>[
          CupertinoDialogAction(
            // isDefaultAction
            // 작업의 기본 값
            // 텍스트 색상을 파란색으로, 굵게 표시
            isDefaultAction: true,
            onPressed: () => Navigator.pop(context),
            child: const Text('닫기'),
          ),
        ],
      ),
    );
  }

  // 서비스 등록
  void registerService() {
    mPush.registerService().then((value) {
      setResult("registerService", value.toString());
    });
  }

  // 사용자 등록
  void registerUser() {
    mPush.registerUser({
      "cuid": "tester",
      "name": "testerName"
    }).then((value) {
      setResult("registerUser", value.toString());
    });
  }

  // 서비스 & 사용자 등록
  void registerServiceAndUser() {
    mPush.registerServiceAndUser({
      "cuid": "tester",
      "name": "testerName"
    }).then((value) {
      setResult("registerServiceAndUser", value.toString());
    });
  }

  // 서비스 해지
  void unregisterService() {
    mPush.unregisterService().then((value) {
      setResult("unregisterService", value.toString());
    });
  }

  // 사용자 해지
  void unregisterUser() {
    mPush.unregisterUser().then((value) {
      setResult("unregisterUser", value.toString());
    });
  }

  // 메시지 읽음 확인
  void readNotification() {
    if (_noti != null) {
      mPush.readNotification({
        "notification": _noti
      }).then((value) {
        setResult("readNotification", value.toString());
      });
    } else {
      setResult("readNotification", "메시지를 수신후 테스트해주세요");
    }
  }

  // 서버 뱃지 카운트 변경
  void remoteSetBadge() {
    mPush.remoteSetBadge({
      "badge": 10
    }).then((value) {
      setResult("remoteSetBadge", value.toString());
    });
  }

  // 테스트 메시지 발송
  void sendNotification() {
    mPush.sendNotification({
      "cuid": 'tester',
      "serviceCode": 'ALL',
      "message": '기본알림입니다.'
    }).then((value) {
      setResult("sendNotification", value.toString());
    });
  }

  // 로컬 뱃지 카운트
  void notificationCenterGetBadge() {
    mPush.notificationCenterGetBadgeNumber().then((value) {
      setResult("notificationCenterGetBadgeNumber", value.toString());
    });
  }

  // 로컬 뱃지 카운트
  void notificationCenterSetBadge() {
    mPush.notificationCenterSetBadgeNumber({
      "badge": 10
    }).then((value) {
      setResult("notificationCenterSetBadgeNumber", value.toString());
    });
  }

  // 테스트 메시지 발송
  void registerGroup() {
    mPush.registerGroup({
      "groupNum": "10"
    }).then((value) {
      setResult("registerGroup", value.toString());
    });
  }

  // 테스트 메시지 발송
  void unregisterGroup() {
    mPush.unregisterGroup({
      "groupNum": "10"
    }).then((value) {
      setResult("unregisterGroup", value.toString());
    });
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Push Sample',
      home: Scaffold(
        appBar: AppBar(
          backgroundColor: Theme.of(context).colorScheme.inversePrimary,
          title: const Text("Flutter Push Sample"),
        ),
        body: Column(
          children: [
            Container(
              decoration: BoxDecoration(
                border: Border.all(
                  width: 2,
                  color: Colors.black,
                ),
              ), //  POINT: BoxDecoration
              width: double.infinity, height: 200,
              padding: const EdgeInsets.all(15),
              child: SingleChildScrollView(
                child: Text(
                  _result,
                ),
              ),
            ),
            Expanded(
              child: Container(
                width: double.infinity,
                child: SingleChildScrollView(
                  child: Padding(
                    padding: const EdgeInsets.all(15.0),
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        const Text(
                          "1. 서버에 사용자 등록을 수행",
                        ),
                        OutlinedButton(
                          child: const Text('서비스 사용자 등록'),
                          onPressed: () {
                            registerServiceAndUser();
                          },
                        ),
                        const Text(
                          "2. 테스트 메시지 발송",
                        ),
                        OutlinedButton(
                          child: const Text('메시지 발송'),
                          onPressed: () {
                            sendNotification();
                          },
                        ),
                        const Text(
                          "3. 서버에 메시지 확인했음을 전송",
                        ),
                        OutlinedButton(
                          child: const Text('메시지 읽음'),
                          onPressed: () {
                            readNotification();
                          },
                        ),
                        const Text(
                          "4. 등록된 사용자 해제",
                        ),
                        OutlinedButton(
                          child: const Text('서비스 해제'),
                          onPressed: () {
                            unregisterService();
                          },
                        ),
                        const Padding(
                          padding: EdgeInsets.all(10.0),
                          child: Text(
                            "- iOS Only API",
                          ),
                        ),
                        const Text(
                          "서버에서 관리되는 뱃지값 변경",
                        ),
                        OutlinedButton(
                          child: const Text('badge Count 변경'),
                          onPressed: () {
                            remoteSetBadge();
                          },
                        ),
                        const Text(
                          "클라이언트에서 관리되는 뱃지값 변경",
                        ),
                        OutlinedButton(
                          child: const Text('setBadge'),
                          onPressed: () {
                            notificationCenterSetBadge();
                          },
                        ),
                        OutlinedButton(
                          child: const Text('getBadge'),
                          onPressed: () {
                            notificationCenterGetBadge();
                          },
                        ),
                      ],
                    ),
                  ),
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }
}
