#import "MPushPlugin.h"
#import "PushReceiver.h"

@implementation NSDictionary (MJson)

- (NSString *)jsonString {
    NSData *jsondata = [NSJSONSerialization dataWithJSONObject:self options:NSJSONWritingPrettyPrinted error:nil];
    return [[NSString alloc] initWithData:jsondata encoding:NSUTF8StringEncoding];
}

@end

@interface MPushPlugin() <PushManagerDelegate>

@end


@implementation MPushPlugin
+ (MPushPlugin *)getInstace {
    static MPushPlugin *sharedInstance;
    @synchronized(self) {
        if (!sharedInstance) {
            sharedInstance = [[MPushPlugin alloc] init];
            return sharedInstance;
        }
    }
    return sharedInstance;
}

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    [MPushPlugin getInstace].channel = [FlutterMethodChannel methodChannelWithName:@"m_push_plugin" binaryMessenger:[registrar messenger]];
    [registrar addMethodCallDelegate:[MPushPlugin getInstace] channel:[MPushPlugin getInstace].channel];
    [registrar addApplicationDelegate:[MPushPlugin getInstace]];
}

- (NSString *)getInvalidMessage:(NSDictionary *)options withValidClasses:(NSDictionary *)validClasses {
    
    for ( NSString *key in validClasses ) {
        Class validClass = [validClasses objectForKey:key];
        id parameter = [options objectForKey:key];
        
        if ( parameter == nil ) {
            return [NSString stringWithFormat:@"key(%@) is null", key];
        }
        
        if ( ![parameter isKindOfClass:validClass] ) {
            return [NSString stringWithFormat:@"key(%@) is invalid type", key];
        }
        
        if ( [validClass isKindOfClass:[NSString class]] && [[parameter stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]] isEqualToString:@""]) {
            return [NSString stringWithFormat:@"key(%@) is invalid", key];
        }
        else if ( [validClass isKindOfClass:[NSDictionary class]] && [[parameter allKeys] count] == 0 ) {
            return [NSString stringWithFormat:@"key(%@) is invalid", key];
        }
        else if ( [validClass isKindOfClass:[NSArray class]] && [parameter count] == 0 ) {
            return [NSString stringWithFormat:@"key(%@) is invalid", key];
        }
    }
    
    return nil;
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    if ([@"getPlatformVersion" isEqualToString:call.method]) {
        result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
    }
    
    // notificationCenterGetBadgeNumber
    else if ([@"notificationCenterGetBadgeNumber" isEqualToString:call.method]) {
        NSNumber *badgeNumber = [[[PushManager defaultManager] notificationCenter] badgeNumber];
        result([badgeNumber stringValue]);
    }
    
    // notificationCenterSetBadgeNumber
    else if ([@"notificationCenterSetBadgeNumber" isEqualToString:call.method]) {
        int badge = [[call.arguments objectForKey:@"badge"] intValue];
        [[[PushManager defaultManager] notificationCenter] setBadgeNumber:@(badge)];
        NSNumber *badgeNumber = [[[PushManager defaultManager] notificationCenter] badgeNumber];
        result([badgeNumber stringValue]);
    }

    //registerService
    else if ([@"registerService" isEqualToString:call.method]) {
        [[PushManager defaultManager] registerService:nil completionHandler:^(BOOL success) {
            if(success)
            {
                result(@{@"status":@"SUCCESS"});
            }
            else
            {
                result(@{@"status":@"FAIL", @"error": @"register service is fail"});
            }
        }];
    }
    
    //registerUser
    else if ([@"registerUser" isEqualToString:call.method]) {
        NSString *invalidMessage = [self getInvalidMessage:call.arguments withValidClasses:@{
            @"cuid": [NSString class],
            @"name": [NSString class]
        }];
        
        if ( invalidMessage != nil ) {
            result(@{@"status":@"FAIL", @"error": invalidMessage});
            return;
        }
        
        NSString *cuid = [call.arguments objectForKey:@"cuid"];
        NSString *cname = [call.arguments objectForKey:@"name"];

        [[PushManager defaultManager] registerUser:nil clientUID:cuid clientName:cname completionHandler:^(BOOL success) {
            if(success)
            {
                result(@{@"status":@"SUCCESS"});
            }
            else
            {
                result(@{@"status":@"FAIL", @"error": @"register user is fail"});
            }
        }];
    }
    
    // registerServiceAndUser
    else if ([@"registerServiceAndUser" isEqualToString:call.method]) {
        NSString *invalidMessage = [self getInvalidMessage:call.arguments withValidClasses:@{
            @"cuid": [NSString class],
            @"name": [NSString class]
        }];
        
        if ( invalidMessage != nil ) {
            result(@{@"status":@"FAIL", @"error": invalidMessage});
            return;
        }
        
        NSString *cuid = [call.arguments objectForKey:@"cuid"];
        NSString *cname = [call.arguments objectForKey:@"name"];
        
        [[PushManager defaultManager] registerServiceAndUser:nil clientUID:cuid clientName:cname completionHandler:^(BOOL success) {
            if(success)
            {
                result(@{@"status":@"SUCCESS"});
            }
            else
            {
                result(@{@"status":@"FAIL", @"error": @"register serviceAndUser is fail"});
            }
        }];
    }
    
    // unRegisterService
    else if ([@"unregisterService" isEqualToString:call.method]) {
        [[PushManager defaultManager] unregisterService:nil completionHandler:^(BOOL success) {
            if(success)
            {
                result(@{@"status":@"SUCCESS"});
            }
            else
            {
                result(@{@"status":@"FAIL", @"error": @"unregister service is fail"});
            }
        }];
    }
    
    // unRegisterUser
    else if ([@"unregisterUser" isEqualToString:call.method]) {
        [[PushManager defaultManager] unregisterUser:nil completionHandler:^(BOOL success) {
            if(success)
            {
                result(@{@"status":@"SUCCESS"});
            }
            else
            {
                result(@{@"status":@"FAIL", @"error": @"unregister user is fail"});
            }
        }];
    }
    
    // readNotification
    else if ([@"readNotification" isEqualToString:call.method]) {
        NSDictionary *noti = [call.arguments objectForKey:@"notification"];
        [[PushManager defaultManager] read:nil notification:noti completionHandler:^(BOOL success) {
            if(success)
            {
                result(@{@"status":@"SUCCESS"});
            }
            else
            {
                result(@{@"status":@"FAIL", @"error": @"readNotification is fail"});
            }
        }];
    }
    
    // remoteSetBadgeNumber
    else if ([@"remoteSetBadgeNumber" isEqualToString:call.method]) {
        int badge = [[call.arguments objectForKey:@"badge"] intValue];
        [[PushManager defaultManager] update:nil badge:@(badge) completionHandler:^(BOOL success) {
            if(success)
            {
                result(@{@"status":@"SUCCESS"});
            }
            else
            {
                result(@{@"status":@"FAIL", @"error": @"remoteSetBadgeNumber is fail"});
            }
        }];
    }
    
    // sendNotification
    else if ([@"sendNotification" isEqualToString:call.method]) {
        NSString *invalidMessage = [self getInvalidMessage:call.arguments withValidClasses:@{
            @"cuid": [NSString class],
            @"serviceCode": [NSString class],
            @"message": [NSString class]
        }];
        
        if ( invalidMessage != nil ) {
            result(@{@"status":@"FAIL", @"error": invalidMessage});
            return;
        }
        
        NSString *cuid = [call.arguments objectForKey:@"cuid"];
        NSString *serviceCode = [call.arguments objectForKey:@"serviceCode"];
        NSString *message = [call.arguments objectForKey:@"message"];
        NSString *ext = [call.arguments objectForKey:@"ext"];

        [[PushManager defaultManager] send:nil clientUID:cuid message:message serviceCode:serviceCode ext:ext completionHandler:^(BOOL success) {
            if(success)
            {
                result(@{@"status":@"SUCCESS"});
            }
            else
            {
                result(@{@"status":@"FAIL", @"error": @"sending notification is fail"});
            }
        }];
    }
    
    //registerGroup
    else if ([@"registerGroup" isEqualToString:call.method]) {
        NSString *groupNum = [call.arguments objectForKey:@"groupNum"];
        [[PushManager defaultManager] registerGroup:nil groupSequenceNumber:groupNum completionHandler:^(BOOL success) {
            if(success)
            {
                result(@{@"status":@"SUCCESS"});
            }
            else
            {
                result(@{@"status":@"FAIL", @"error": @"registerGroup is fail"});
            }
        }];
    }

    //unregisterGroup
    else if ([@"unregisterGroup" isEqualToString:call.method]) {
        NSString *groupNum = [call.arguments objectForKey:@"groupNum"];
        [[PushManager defaultManager] unregisterGroup:nil groupSequenceNumber:groupNum completionHandler:^(BOOL success) {
            if(success)
            {
                result(@{@"status":@"SUCCESS"});
            }
            else
            {
                result(@{@"status":@"FAIL", @"error": @"unregisterGroup is fail"});
            }
        }];
    }

    else if ([@"setEnabled" isEqualToString:call.method]) {
        BOOL enabled = call.arguments;
        [[PushManager defaultManager] setEnabled:enabled];
    }

    else if ([@"onNotification" isEqualToString:call.method]) {
    }
    
    else {
        result(FlutterMethodNotImplemented);
    }
}

#pragma mark - ApplicationDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    [[PushManager defaultManager] application:application didFinishLaunchingWithOptions:launchOptions];
    PushReceiver *receiver = [[PushReceiver alloc] init];
    [[PushManager defaultManager] initilaizeWithDelegate:receiver];
    [UNUserNotificationCenter currentNotificationCenter].delegate = self;
    
    return YES;
}

- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken
{
    [[PushManager defaultManager] application:application didRegisterForRemoteNotificationsWithDeviceToken:deviceToken];
}

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo
{
    [[PushManager defaultManager] application:application didReceiveRemoteNotification:userInfo];
}

- (void)manager:(PushManager *)manager didReceiveUserNotification:(NSDictionary *)userInfo status:(NSString *)status messageUID:(NSString *)messageUID
{
    [self.channel invokeMethod:@"didReceiveUserNotification" arguments:@{@"status":status, @"payload":userInfo, @"type":@"APNS", @"messageUID": messageUID} result:^(id result) {
    }];
}

- (void)userNotificationCenter:(UNUserNotificationCenter *)center willPresentNotification:(UNNotification *)notification withCompletionHandler:(void (^)(UNNotificationPresentationOptions))completionHandler
{
    [[PushManager defaultManager] userNotificationCenter:center willPresentNotification:notification withCompletionHandler:completionHandler];
}

- (void)manager:(PushManager *)manager userNotificationCenter:(UNUserNotificationCenter *)center willPresentNotification:(UNNotification *)notification withCompletionHandler:(void (^)(UNNotificationPresentationOptions))completionHandler
{
    completionHandler( UNNotificationPresentationOptionAlert | UNNotificationPresentationOptionSound | UNNotificationPresentationOptionBadge);
}

@end
