import 'package:flutter/services.dart';

typedef void DidReceiveNotification(Map notification);

class MPushPlugin {
  MethodChannel methodChannel = const MethodChannel('m_push_plugin');

  DidReceiveNotification? _didReceiveNotification;

  MPushPlugin() {
    this.methodChannel.setMethodCallHandler(_handleMethod);
    setEnabled(true);
  }

  void setDidReceiveNotification(DidReceiveNotification handler) {
    _didReceiveNotification = handler;
  }

  //푸시로 앱 시작시 사용
  Future<Map> getOnNotification() async {
    print("getOnNotification");
    final Map result = await methodChannel.invokeMethod("onNotification");
    return result;
  }

  Future<Map> registerService() async {
    final Map result = await methodChannel.invokeMethod('registerService');
    return result;
  }

  Future<Map> registerUser(Map param) async {
    final Map result = await methodChannel.invokeMethod('registerUser', param);
    return result;
  }

  Future<Map> registerServiceAndUser(Map param) async {
    final Map result = await methodChannel.invokeMethod('registerServiceAndUser', param);
    return result;
  }

  Future<Map> unregisterService() async {
    final Map result = await methodChannel.invokeMethod('unregisterService');
    return result;
  }

  Future<Map> unregisterUser() async {
    final Map result = await methodChannel.invokeMethod('unregisterUser');
    return result;
  }

  Future<Map> readNotification(Map param) async {
    final Map result = await methodChannel.invokeMethod('readNotification', param);
    return result;
  }

  Future<Map> remoteSetBadge(Map param) async {
    final Map result = await methodChannel.invokeMethod('remoteSetBadgeNumber', param);
    return result;
  }

  Future<Map> sendNotification(Map param) async {
    final Map result = await methodChannel.invokeMethod('sendNotification', param);
    return result;
  }

  Future<int> notificationCenterGetBadgeNumber() async {
    final result = await methodChannel.invokeMethod('notificationCenterGetBadgeNumber');
    return int.parse(result!);
  }

  Future<int> notificationCenterSetBadgeNumber(Map param) async {
    final result = await methodChannel.invokeMethod('notificationCenterSetBadgeNumber', param);
    return int.parse(result!);
  }

  Future<Map> registerGroup(Map param) async {
    final Map result = await methodChannel.invokeMethod('registerGroup', param);
    return result;
  }

  Future<Map> unregisterGroup(Map param) async {
    final Map result = await methodChannel.invokeMethod('unregisterGroup', param);
    return result;
  }

  Future<void> setEnabled(bool param) async {
    methodChannel.invokeMethod('setEnabled', param);
  }

  Future<Null> _handleMethod(MethodCall call) async {
    _didReceiveNotification!(call.arguments);
    return null;
  }
}
